@@ -4,10 +4,15 @@ export type IColTypeTuple = {
     col_type: string;
 };
 
+export type Warning = {
+    warnMsg: string;
+}
+
 export type IDFColMap = {
     [key: string]: {
         columns: IColTypeTuple[];
         python_id: string;
+        warnings: Warning[]
     };
 };
 
@@ -22,6 +27,7 @@ export type IDFProfileWState = IDFProfileData & IDFProfileState
 export type IDFProfileState = {
     lastUpdatedTime: number;
     isPinned: boolean;
+    warnings: Warning[]
 }
 
 export type IDFProfileData = {
@@ -49,6 +55,12 @@ export type IColMeta = {
     nullCount: number;
 };
 
+export type IStringMeta = {
+    minLength: number;
+    maxLength: number;
+    meanLength: number;
+};
+
 export type ColumnProfileData = {
     name: string;
     type: string;
@@ -63,6 +75,7 @@ export interface ColumnSummary {
     histogram?: IHistogram;
     statistics?: IQuantMeta;
     timeSummary?: TimeColumnSummary;
+    stringSummary?: IStringMeta;
 }
 
 // TODO this could be subclass but then need better type guards
