@@ -29,13 +29,12 @@ export function formatNumeric(type: string, value) {
 }
 
 
-function formatInt(v) {
+export function formatInt(v) {
     return format('d')(v)
 }
 
-function formatFloat(v) {
+export function formatFloat(v) {
     if (Math.abs(v) >= 1) {
-        // return parseFloat(value.toFixed(1));
         return v.toFixed(1);
     }
 
@@ -63,8 +62,6 @@ export function removeTimezoneOffset(dt) {
     return new Date(dt.getTime() + dt.getTimezoneOffset() * 60000);
 }
 
-//export const formatPercentage = format('.2%');
-
 export const standardTimestampFormat = (v, type = 'TIMESTAMP') => {
     let fmt = timeFormat('%b %d, %Y %I:%M:%S');
     if (type === 'DATE') {
