@@ -11,9 +11,11 @@ import type {
 } from '../common/exchangeInterfaces';
 import {
     NUMERICS,
-    TIMESTAMPS
+    TIMESTAMPS,
+    CATEGORICALS
 } from '../components/data-types/pandas-data-types';
 import _ from 'lodash';
+import type { Logger } from '../logger/Logger';
 
 export class ProfileModel {
 
@@ -25,6 +27,7 @@ export class ProfileModel {
     private _name: Writable<string> = writable(undefined)
     private _varsInCurrentCell: Writable<string[]> = writable([])
     private _currentOutputName: string;
+    private _logger;
 
     constructor(session: ISessionContext) {
         this._executor = new PythonPandasExecutor(session)
@@ -71,6 +74,14 @@ export class ProfileModel {
         return undefined
     }
 
+    get logger(): Logger {
+        return this._logger
+    }
+
+    addLogger(logger: Logger) {
+        this._logger = logger
+    }
+
     public async connectNotebook(notebook: NotebookAPI) {
         console.log('Connecting notebook to ProfilePanel');
         this._notebook = notebook;
@@ -80,6 +91,7 @@ export class ProfileModel {
         await this.executor.session.ready;
 
         this._name.set(this.executor.session.name)
+        this.logger.log('ProfileModel.connectNotebook', { notebookName: this.executor.session.name })
         // have to do this as arrow function or else this doesnt work
         this._notebook.changed.connect((sender, value) => {
             // when cell is run, update data
@@ -104,6 +116,7 @@ export class ProfileModel {
         this.executor.session.session?.kernel.statusChanged.connect((_, status) => {
             if (status.endsWith('restarting')) {
                 this.resetData();
+                this.logger.log('ProfileModel.kernelRestarted', { notebookName: this.name })
             }
         });
     }
@@ -114,11 +127,28 @@ export class ProfileModel {
 
     public async updateRootData() {
         this._loadingNewData.set(true)
-        const alldf = await this.executor.getAllDataFrames(this.currentOutputName);
+        let alldf = await this.executor.getAllDataFrames(this.currentOutputName);
+
 
         // only update if we have detected dataframes
         if (!_.isEmpty(alldf)) {
-            const colPromise = this.fetchColumnPromises(alldf);
+
+            // more cells might have executed, so filter to only dataframes in current cell
+            const currentDfs = Object.keys(alldf).filter((key: string) => {
+                if (key == this.currentOutputName) {
+                    return true
+                } else if (key.charAt(0) == '_') {
+                    return false
+                }
+                return true
+            }).reduce((obj, key) => {
+                return {
+                    ...obj,
+                    [key]: alldf[key]
+                }
+            }, {})
+
+            const colPromise = this.fetchColumnPromises(currentDfs);
 
             colPromise.then(result => {
 
@@ -138,11 +168,19 @@ export class ProfileModel {
 
                                 if (!_.isEqual(currentShapeAndProfile, newShapeAndProfile)) {
                                     result[dfName].lastUpdatedTime = Date.now()
+                                    this.logger.log('ProfileModel.updateData', { dfName })
                                 }
                             }
                         }
                     }
 
+                    // copy warnings over from alldf to result
+                    for (const [dfName, dfInfo] of Object.entries(alldf)) {
+                        if (dfName in result) {
+                            result[dfName].warnings = dfInfo.warnings
+                        }
+                    }
+
                     return result
                 });
                 this._loadingNewData.set(false);
@@ -180,6 +218,7 @@ export class ProfileModel {
 
         alldf_names.forEach((dfName, index) => {
             colProfileMap[dfName] = resolved_profiles[index];
+            colProfileMap[dfName].warnings = dfColMap[dfName].warnings
         });
         return colProfileMap;
     }
@@ -245,6 +284,10 @@ export class ProfileModel {
                     };
 
                     cd.summary.timeSummary = timeSummary;
+                } else if (CATEGORICALS.has(col_type)) {
+                    const stringSummary = await this.executor.getStringStats(dfName, col_name);
+
+                    cd.summary.stringSummary = stringSummary;
                 }
             }
 
@@ -252,7 +295,7 @@ export class ProfileModel {
             resultData.push(cd);
         }
 
-        return { profile: resultData, shape, dfName, lastUpdatedTime: Date.now(), isPinned: false };
+        return { profile: resultData, shape, dfName, lastUpdatedTime: Date.now(), isPinned: false, warnings: [] };
     }
 
 }
