@@ -8,8 +8,11 @@ import type {
     IHistogram,
     ValueCount,
     Interval,
-    TimeBin
+    TimeBin,
+    IStringMeta
 } from '../../common/exchangeInterfaces';
+import _ from 'lodash';
+
 
 type ExecResult = { content: string[]; exec_count: number };
 
@@ -159,11 +162,18 @@ export class PythonPandasExecutor {
 
                     for (let index = 0; index < vars_DF.length; index++) {
                         const dfName = vars_DF[index];
-                        const columnTuples = await this.getColumns(dfName);
+                        let { columnsWithTypes, duplicates } = await this.getColumns(dfName);
+                        const warnings = []
+
+                        if (!_.isEmpty(duplicates)) {
+                            warnings.push({ warnMsg: `All columns must have unique names. The following columns are excluded: ${duplicates.join(', ')}.` })
+                            columnsWithTypes = columnsWithTypes.filter(col => !duplicates.includes(col.col_name))
+                        }
 
                         dfColMap[dfName] = {
-                            columns: columnTuples,
-                            python_id: python_ids[index]
+                            columns: columnsWithTypes,
+                            python_id: python_ids[index],
+                            warnings
                         };
                     }
                     return dfColMap;
@@ -231,26 +241,38 @@ export class PythonPandasExecutor {
         }
     }
 
-    private async getColumns(dfName: string): Promise<IColTypeTuple[]> {
+    private async getColumns(dfName: string): Promise<{ columnsWithTypes: IColTypeTuple[], duplicates: string[] }> {
         /*
         dfName is variable that is pd.DataFrame
         Returns array of "True" or "False" if that variable is a pandas dataframe
         */
+        const code = `digautoprofiler.getColumns(${dfName})`
         try {
-            const res = await this.executePythonAP(`digautoprofiler.getColumns(${dfName})`);
+            const res = await this.executePythonAP(code);
             const content = res['content']; // might be null
             const json_res = JSON.parse(content?.join(""));
 
             const columnsWithTypes: IColTypeTuple[] = []
 
-            for (const [key, value] of Object.entries(json_res)) {
-                columnsWithTypes.push({ "col_name": key, "col_type": (value as string) })
+            let uniqueNames = new Set<string>()
+            let duplicatedNames = new Set<string>()
+
+            for (const item of json_res) {
+                columnsWithTypes.push({
+                    col_name: item["colName"],
+                    col_type: item["type"]
+                })
+                if (uniqueNames.has(item["colName"])) {
+                    duplicatedNames.add(item["colName"])
+                } else {
+                    uniqueNames.add(item["colName"])
+                }
             }
 
-            return columnsWithTypes
+            return { columnsWithTypes, duplicates: Array.from(duplicatedNames) }
         } catch (error) {
-            console.warn('[Error caught] in getColumns', error);
-            return [];
+            console.warn(`[Error caught] in getColumns executing code: ${code}`, error);
+            return { columnsWithTypes: [], duplicates: [] };
         }
     }
 
@@ -258,8 +280,9 @@ export class PythonPandasExecutor {
         /*
         returns tuple array [length, width]
         */
+        const code = `digautoprofiler.getShape(${dfName})`
         try {
-            const res = await this.executePythonAP(`digautoprofiler.getShape(${dfName})`);
+            const res = await this.executePythonAP(code);
             const content = res['content'];
             const shapeString = content.join("");
             return shapeString
@@ -267,7 +290,7 @@ export class PythonPandasExecutor {
                 .split(',')
                 .map(x => parseFloat(x));
         } catch (error) {
-            console.warn('[Error caught] in getShape', error);
+            console.warn(`[Error caught] in getShape executing: ${code}`, error);
             return [undefined, undefined];
         }
     }
@@ -276,8 +299,9 @@ export class PythonPandasExecutor {
         dfName: string,
         colName: string
     ): Promise<IQuantMeta> {
+        const code = `digautoprofiler.getQuantMeta(${dfName}, "${replaceSpecial(colName)}")`
         try {
-            const res = await this.executePythonAP(`digautoprofiler.getQuantMeta(${dfName}, "${replaceSpecial(colName)}")`);
+            const res = await this.executePythonAP(code);
             const content = res['content']; // might be null
             const json_res = JSON.parse(content?.join("").replace(/'/g, '')); // remove single quotes bc not JSON parseable
 
@@ -290,7 +314,7 @@ export class PythonPandasExecutor {
                 mean: parseFloat(json_res['mean'])
             };
         } catch (error) {
-            console.warn('[Error caught] in getQuantMeta', error);
+            console.warn(`[Error caught] in getQuantMeta executing: ${code} `, error);
             return {
                 min: undefined,
                 q25: undefined,
@@ -306,15 +330,16 @@ export class PythonPandasExecutor {
         dfName: string,
         colName: string
     ): Promise<IColMeta> {
+        const code = `digautoprofiler.getColMeta(${dfName}, "${replaceSpecial(colName)}")`;
         try {
-            const res = await this.executePythonAP(`digautoprofiler.getColMeta(${dfName}, "${replaceSpecial(colName)}")`);
+            const res = await this.executePythonAP(code);
             const content = res['content'];
             return {
                 numUnique: parseInt(content[0]),
                 nullCount: parseInt(content[1])
             };
         } catch (error) {
-            console.warn('[Error caught] in getColMeta', error);
+            console.warn(`[Error caught] in getColMeta executing: ${code}`, error);
             return {
                 numUnique: undefined,
                 nullCount: undefined
@@ -322,13 +347,37 @@ export class PythonPandasExecutor {
         }
     }
 
+    public async getStringStats(
+        dfName: string,
+        colName: string
+    ): Promise<IStringMeta> {
+        const code = `digautoprofiler.getStringStats(${dfName}, "${replaceSpecial(colName)}")`;
+        try {
+            const res = await this.executePythonAP(code);
+            const content = res['content'];
+            return {
+                minLength: parseInt(content[0]),
+                maxLength: parseInt(content[1]),
+                meanLength: parseFloat(content[2]),
+            };
+        } catch (error) {
+            console.warn(`[Error caught] in getStringStats executing: ${code}`, error);
+            return {
+                minLength: undefined,
+                maxLength: undefined,
+                meanLength: undefined,
+            };
+        }
+    }
+
     public async getValueCounts(
         dfName: string,
         colName: string,
         n = 10
     ): Promise<ValueCount[]> {
+        const code = `digautoprofiler.getValueCounts(${dfName}, "${replaceSpecial(colName)}", ${n})`;
         try {
-            const res = await this.executePythonAP(`digautoprofiler.getValueCounts(${dfName}, "${replaceSpecial(colName)}", ${n})`);
+            const res = await this.executePythonAP(code);
             const data: ValueCount[] = [];
             const content = res['content']; // might be null
             const json_res = JSON.parse(content?.join("").replace(/'/g, '')); // remove single quotes bc not JSON parseable
@@ -337,7 +386,7 @@ export class PythonPandasExecutor {
             });
             return data;
         } catch (error) {
-            console.warn('[Error caught] in getValueCounts', error);
+            console.warn(`[Error caught] in getValueCounts executing: ${code}`, error);
             return [];
         }
     }
@@ -347,9 +396,11 @@ export class PythonPandasExecutor {
         colName: string,
         maxbins = 20
     ): Promise<IHistogram> {
+        const code = `digautoprofiler.getQuantBinnedData(${dfName}, "${replaceSpecial(colName)}", ${maxbins})`
         try {
-            const res = await this.executePythonAP(`digautoprofiler.getQuantBinnedData(${dfName}, "${replaceSpecial(colName)}", ${maxbins})`);
+            const res = await this.executePythonAP(code);
             const content = res['content'];
+
             const data: IHistogram = [];
             const json_res = JSON.parse(content.join().replace(/'/g, '')); // remove single quotes bc not JSON parseable
 
@@ -365,7 +416,7 @@ export class PythonPandasExecutor {
             });
             return data;
         } catch (error) {
-            console.warn('[Error caught] in getQuantBinnedData', error);
+            console.warn(`[Error caught] in getQuantBinnedData executing: ${code}`, error);
             return [];
         }
     }
@@ -375,8 +426,9 @@ export class PythonPandasExecutor {
         colName: string,
         maxbins = 200
     ): Promise<{ timebin: TimeBin[], histogram: IHistogram }> {
+        const code = `digautoprofiler.getTempBinnedData(${dfName}, "${replaceSpecial(colName)}", ${maxbins})`;
         try {
-            const res = await this.executePythonAP(`digautoprofiler.getTempBinnedData(${dfName}, "${replaceSpecial(colName)}", ${maxbins})`);
+            const res = await this.executePythonAP(code);
             const content = res['content'];
             const timebinData: TimeBin[] = [];
             const histogram: IHistogram = []
@@ -413,7 +465,7 @@ export class PythonPandasExecutor {
             });
             return { timebin: timebinData, histogram: histogram };
         } catch (error) {
-            console.warn('[Error caught] in getTempBinnedData', error);
+            console.warn(`[Error caught] in getTempBinnedData executing: ${code}`, error);
             return { timebin: [], histogram: [] };
         }
     }
@@ -429,8 +481,9 @@ export class PythonPandasExecutor {
         dfName: string,
         colName: string
     ): Promise<Interval> {
+        const code = `digautoprofiler.getTempInterval(${dfName}, "${replaceSpecial(colName)}")`;
         try {
-            const res = await this.executePythonAP(`digautoprofiler.getTempInterval(${dfName}, "${replaceSpecial(colName)}")`);
+            const res = await this.executePythonAP(code);
             const content = res['content'];
             return {
                 months: 0,
@@ -438,7 +491,7 @@ export class PythonPandasExecutor {
                 micros: 0
             };
         } catch (error) {
-            console.warn('[Error caught] in getColMeta', error);
+            console.warn(`[Error caught] in getColMeta executing ${code}`, error);
             return {
                 months: 0,
                 days: 0,
@@ -448,9 +501,10 @@ export class PythonPandasExecutor {
     }
 
     public async getVariableNamesInPythonStr(codeString: string): Promise<string[]> {
+        const formattedCode = codeString.replace(/"/g, '\\"');
+        const code = `digautoprofiler.getVariableNamesInPythonStr("""${formattedCode}""")`;
         try {
-            const formattedCode = codeString.replace(/"/g, '\\"');
-            const res = await this.executePythonAP(`digautoprofiler.getVariableNamesInPythonStr("""${formattedCode}""")`)
+            const res = await this.executePythonAP(code)
             let content = res["content"].join("")
             content = content.replace(/'/g, '"') // replace single quotes
             content = content.replace('{', '[')
